@@ -37,6 +37,7 @@ DEFAULT_CONTAINER_FORMAT = 'bare'
 DEFAULT_DISK_FORMAT = 'raw'
 DISK_CHOICES = ["ami", "ari", "aki", "vhd", "vmdk", "raw", "qcow2", "vhdx",
                 "vdi", "iso", "ploop"]
+MEMBER_STATUS_CHOICES = ["accepted", "pending", "rejected", "all"]
 
 
 LOG = logging.getLogger(__name__)
@@ -86,7 +87,7 @@ class AddProjectToImage(command.ShowOne):
         parser.add_argument(
             "project",
             metavar="<project>",
-            help=_("Project to associate with image (name or ID)"),
+            help=_("Project to associate with image (ID)"),
         )
         common.add_project_domain_option_to_parser(parser)
         return parser
@@ -515,7 +516,8 @@ class ListImage(command.Lister):
             '--property',
             metavar='<key=value>',
             action=parseractions.KeyValueAction,
-            help=_('Filter output based on property'),
+            help=_('Filter output based on property '
+                   '(repeat option to filter on multiple properties)'),
         )
         parser.add_argument(
             '--name',
@@ -529,6 +531,16 @@ class ListImage(command.Lister):
             default=None,
             help=_("Filter images based on status.")
         )
+        parser.add_argument(
+            '--member-status',
+            metavar='<member-status>',
+            default=None,
+            type=lambda s: s.lower(),
+            choices=MEMBER_STATUS_CHOICES,
+            help=(_("Filter images based on member status. "
+                    "The supported options are: %s. ") %
+                  ', '.join(MEMBER_STATUS_CHOICES))
+        )
         parser.add_argument(
             '--tag',
             metavar='<tag>',
@@ -594,6 +606,8 @@ class ListImage(command.Lister):
             kwargs['name'] = parsed_args.name
         if parsed_args.status:
             kwargs['status'] = parsed_args.status
+        if parsed_args.member_status:
+            kwargs['member_status'] = parsed_args.member_status
         if parsed_args.tag:
             kwargs['tag'] = parsed_args.tag
         if parsed_args.long:
@@ -643,14 +657,13 @@ class ListImage(command.Lister):
                 marker = page[-1]['id']
 
         if parsed_args.property:
-            # NOTE(dtroyer): coerce to a list to subscript it in py3
-            attr, value = list(parsed_args.property.items())[0]
-            api_utils.simple_filter(
-                data,
-                attr=attr,
-                value=value,
-                property_field='properties',
-            )
+            for attr, value in parsed_args.property.items():
+                api_utils.simple_filter(
+                    data,
+                    attr=attr,
+                    value=value,
+                    property_field='properties',
+                )
 
         data = utils.sort_items(data, parsed_args.sort, str)
 
