@@ -20,6 +20,7 @@ from openstackclient.i18n import _
 from openstackclient.identity import common as identity_common
 from openstackclient.network import common
 from openstackclient.network import sdk_utils
+from openstackclient.network.v2 import _tag
 
 
 def _format_admin_state(item):
@@ -39,10 +40,11 @@ _formatters = {
     'is_router_external': _format_router_external,
     'availability_zones': utils.format_list,
     'availability_zone_hints': utils.format_list,
+    'tags': utils.format_list,
 }
 
 
-def _get_network_columns(item):
+def _get_columns_network(item):
     column_map = {
         'subnet_ids': 'subnets',
         'is_admin_state_up': 'admin_state_up',
@@ -55,23 +57,22 @@ def _get_network_columns(item):
         'ipv4_address_scope_id': 'ipv4_address_scope',
         'ipv6_address_scope_id': 'ipv6_address_scope',
         'tenant_id': 'project_id',
+        'tags': 'tags',
     }
     return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)
 
 
-def _get_columns(item):
-    columns = list(item.keys())
-    if 'tenant_id' in columns:
-        columns.remove('tenant_id')
-    if 'project_id' not in columns:
-        columns.append('project_id')
-    return tuple(sorted(columns))
+def _get_columns_compute(item):
+    column_map = {
+        'tenant_id': 'project_id',
+    }
+    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)
 
 
-def _get_attrs(client_manager, parsed_args):
+def _get_attrs_network(client_manager, parsed_args):
     attrs = {}
     if parsed_args.name is not None:
-        attrs['name'] = str(parsed_args.name)
+        attrs['name'] = parsed_args.name
     if parsed_args.enable:
         attrs['admin_state_up'] = True
     if parsed_args.disable:
@@ -106,15 +107,19 @@ def _get_attrs(client_manager, parsed_args):
     if parsed_args.description:
         attrs['description'] = parsed_args.description
 
+    # set mtu
+    if parsed_args.mtu:
+        attrs['mtu'] = parsed_args.mtu
+
     # update_external_network_options
     if parsed_args.internal:
         attrs['router:external'] = False
     if parsed_args.external:
         attrs['router:external'] = True
-        if parsed_args.no_default:
-            attrs['is_default'] = False
-        if parsed_args.default:
-            attrs['is_default'] = True
+    if parsed_args.no_default:
+        attrs['is_default'] = False
+    if parsed_args.default:
+        attrs['is_default'] = True
     # Update Provider network options
     if parsed_args.provider_network_type:
         attrs['provider:network_type'] = parsed_args.provider_network_type
@@ -129,11 +134,22 @@ def _get_attrs(client_manager, parsed_args):
         attrs['qos_policy_id'] = _qos_policy.id
     if 'no_qos_policy' in parsed_args and parsed_args.no_qos_policy:
         attrs['qos_policy_id'] = None
-    # Update VLAN Transparency for networks
-    if parsed_args.transparent_vlan:
-        attrs['vlan_transparent'] = True
-    if parsed_args.no_transparent_vlan:
-        attrs['vlan_transparent'] = False
+    # Update DNS network options
+    if parsed_args.dns_domain:
+        attrs['dns_domain'] = parsed_args.dns_domain
+    return attrs
+
+
+def _get_attrs_compute(client_manager, parsed_args):
+    attrs = {}
+    if parsed_args.name is not None:
+        attrs['name'] = parsed_args.name
+    if parsed_args.share:
+        attrs['share_subnet'] = True
+    if parsed_args.no_share:
+        attrs['share_subnet'] = False
+    if parsed_args.subnet is not None:
+        attrs['subnet'] = parsed_args.subnet
     return attrs
 
 
@@ -158,29 +174,13 @@ def _add_additional_network_options(parser):
         dest='segmentation_id',
         help=_("VLAN ID for VLAN networks or Tunnel ID for "
                "GENEVE/GRE/VXLAN networks"))
-
-    vlan_transparent_grp = parser.add_mutually_exclusive_group()
-    vlan_transparent_grp.add_argument(
-        '--transparent-vlan',
-        action='store_true',
-        help=_("Make the network VLAN transparent"))
-    vlan_transparent_grp.add_argument(
-        '--no-transparent-vlan',
-        action='store_true',
-        help=_("Do not make the network VLAN transparent"))
-
-
-def _get_attrs_compute(client_manager, parsed_args):
-    attrs = {}
-    if parsed_args.name is not None:
-        attrs['label'] = str(parsed_args.name)
-    if parsed_args.share:
-        attrs['share_address'] = True
-    if parsed_args.no_share:
-        attrs['share_address'] = False
-    if parsed_args.subnet is not None:
-        attrs['cidr'] = parsed_args.subnet
-    return attrs
+    parser.add_argument(
+        '--dns-domain',
+        metavar='<dns-domain>',
+        dest='dns_domain',
+        help=_("Set DNS domain for this network "
+               "(requires DNS integration extension)")
+    )
 
 
 # TODO(sindhu): Use the SDK resource mapped attribute names once the
@@ -231,6 +231,11 @@ class CreateNetwork(common.NetworkAndComputeShowOne):
             metavar='<description>',
             help=_("Set network description")
         )
+        parser.add_argument(
+            '--mtu',
+            metavar='<mtu>',
+            help=_("Set network mtu")
+        )
         identity_common.add_project_domain_option_to_parser(parser)
         parser.add_argument(
             '--availability-zone-hint',
@@ -284,30 +289,51 @@ class CreateNetwork(common.NetworkAndComputeShowOne):
             metavar='<qos-policy>',
             help=_("QoS policy to attach to this network (name or ID)")
         )
+        vlan_transparent_grp = parser.add_mutually_exclusive_group()
+        vlan_transparent_grp.add_argument(
+            '--transparent-vlan',
+            action='store_true',
+            help=_("Make the network VLAN transparent"))
+        vlan_transparent_grp.add_argument(
+            '--no-transparent-vlan',
+            action='store_true',
+            help=_("Do not make the network VLAN transparent"))
+
         _add_additional_network_options(parser)
+        _tag.add_tag_option_to_parser_for_create(parser, _('network'))
         return parser
 
     def update_parser_compute(self, parser):
         parser.add_argument(
             '--subnet',
             metavar='<subnet>',
+            required=True,
             help=_("IPv4 subnet for fixed IPs (in CIDR notation)")
         )
         return parser
 
     def take_action_network(self, client, parsed_args):
-        attrs = _get_attrs(self.app.client_manager, parsed_args)
-        obj = client.create_network(**attrs)
-        display_columns, columns = _get_network_columns(obj)
+        attrs = _get_attrs_network(self.app.client_manager, parsed_args)
+        if parsed_args.transparent_vlan:
+            attrs['vlan_transparent'] = True
+        if parsed_args.no_transparent_vlan:
+            attrs['vlan_transparent'] = False
+        with common.check_missing_extension_if_error(
+                self.app.client_manager.network, attrs):
+            obj = client.create_network(**attrs)
+
+        # tags cannot be set when created, so tags need to be set later.
+        _tag.update_tags_for_set(client, obj, parsed_args)
+        display_columns, columns = _get_columns_network(obj)
         data = utils.get_item_properties(obj, columns, formatters=_formatters)
         return (display_columns, data)
 
     def take_action_compute(self, client, parsed_args):
         attrs = _get_attrs_compute(self.app.client_manager, parsed_args)
-        obj = client.networks.create(**attrs)
-        columns = _get_columns(obj._info)
-        data = utils.get_dict_properties(obj._info, columns)
-        return (columns, data)
+        obj = client.api.network_create(**attrs)
+        display_columns, columns = _get_columns_compute(obj)
+        data = utils.get_dict_properties(obj, columns)
+        return (display_columns, data)
 
 
 class DeleteNetwork(common.NetworkAndComputeDelete):
@@ -332,8 +358,7 @@ class DeleteNetwork(common.NetworkAndComputeDelete):
         client.delete_network(obj)
 
     def take_action_compute(self, client, parsed_args):
-        network = utils.find_resource(client.networks, self.r)
-        client.networks.delete(network.id)
+        client.api.network_delete(self.r)
 
 
 # TODO(sindhu): Use the SDK resource mapped attribute names once the
@@ -420,7 +445,13 @@ class ListNetwork(common.NetworkAndComputeLister):
             help=_("List networks according to VLAN ID for VLAN networks "
                    "or Tunnel ID for GENEVE/GRE/VXLAN networks")
         )
-
+        parser.add_argument(
+            '--agent',
+            metavar='<agent-id>',
+            dest='agent_id',
+            help=_('List networks hosted by agent (ID only)')
+        )
+        _tag.add_tag_filtering_option_to_parser(parser, _('networks'))
         return parser
 
     def take_action_network(self, client, parsed_args):
@@ -437,6 +468,7 @@ class ListNetwork(common.NetworkAndComputeLister):
                 'provider_network_type',
                 'is_router_external',
                 'availability_zones',
+                'tags',
             )
             column_headers = (
                 'ID',
@@ -449,7 +481,28 @@ class ListNetwork(common.NetworkAndComputeLister):
                 'Network Type',
                 'Router Type',
                 'Availability Zones',
+                'Tags',
             )
+        elif parsed_args.agent_id:
+            columns = (
+                'id',
+                'name',
+                'subnet_ids'
+            )
+            column_headers = (
+                'ID',
+                'Name',
+                'Subnets',
+            )
+            client = self.app.client_manager.network
+            dhcp_agent = client.get_agent(parsed_args.agent_id)
+            data = client.dhcp_agent_hosting_networks(dhcp_agent)
+
+            return (column_headers,
+                    (utils.get_item_properties(
+                        s, columns,
+                        formatters=_formatters,
+                    ) for s in data))
         else:
             columns = (
                 'id',
@@ -510,6 +563,8 @@ class ListNetwork(common.NetworkAndComputeLister):
             args['provider:segmentation_id'] = parsed_args.segmentation_id
             args['provider_segmentation_id'] = parsed_args.segmentation_id
 
+        _tag.get_tag_filtering_args(parsed_args, args)
+
         data = client.networks(**args)
 
         return (column_headers,
@@ -530,10 +585,10 @@ class ListNetwork(common.NetworkAndComputeLister):
             'Subnet',
         )
 
-        data = client.networks.list()
+        data = client.api.network_list()
 
         return (column_headers,
-                (utils.get_item_properties(
+                (utils.get_dict_properties(
                     s, columns,
                     formatters=_formatters,
                 ) for s in data))
@@ -585,6 +640,11 @@ class SetNetwork(command.Command):
             metavar="<description",
             help=_("Set network description")
         )
+        parser.add_argument(
+            '--mtu',
+            metavar="<mtu",
+            help=_("Set network mtu")
+        )
         port_security_group = parser.add_mutually_exclusive_group()
         port_security_group.add_argument(
             '--enable-port-security',
@@ -632,6 +692,7 @@ class SetNetwork(command.Command):
             action='store_true',
             help=_("Remove the QoS policy attached to this network")
         )
+        _tag.add_tag_option_to_parser_for_set(parser, _('network'))
         _add_additional_network_options(parser)
         return parser
 
@@ -639,8 +700,14 @@ class SetNetwork(command.Command):
         client = self.app.client_manager.network
         obj = client.find_network(parsed_args.network, ignore_missing=False)
 
-        attrs = _get_attrs(self.app.client_manager, parsed_args)
-        client.update_network(obj, **attrs)
+        attrs = _get_attrs_network(self.app.client_manager, parsed_args)
+        if attrs:
+            with common.check_missing_extension_if_error(
+                    self.app.client_manager.network, attrs):
+                client.update_network(obj, **attrs)
+
+        # tags is a subresource and it needs to be updated separately.
+        _tag.update_tags_for_set(client, obj, parsed_args)
 
 
 class ShowNetwork(common.NetworkAndComputeShowOne):
@@ -656,15 +723,36 @@ class ShowNetwork(common.NetworkAndComputeShowOne):
 
     def take_action_network(self, client, parsed_args):
         obj = client.find_network(parsed_args.network, ignore_missing=False)
-        display_columns, columns = _get_network_columns(obj)
+        display_columns, columns = _get_columns_network(obj)
         data = utils.get_item_properties(obj, columns, formatters=_formatters)
         return (display_columns, data)
 
     def take_action_compute(self, client, parsed_args):
-        obj = utils.find_resource(
-            client.networks,
-            parsed_args.network,
+        obj = client.api.network_find(parsed_args.network)
+        display_columns, columns = _get_columns_compute(obj)
+        data = utils.get_dict_properties(obj, columns)
+        return (display_columns, data)
+
+
+class UnsetNetwork(command.Command):
+    _description = _("Unset network properties")
+
+    def get_parser(self, prog_name):
+        parser = super(UnsetNetwork, self).get_parser(prog_name)
+        parser.add_argument(
+            'network',
+            metavar="<network>",
+            help=_("Network to modify (name or ID)")
         )
-        columns = _get_columns(obj._info)
-        data = utils.get_dict_properties(obj._info, columns)
-        return (columns, data)
+        _tag.add_tag_option_to_parser_for_unset(parser, _('network'))
+        return parser
+
+    def take_action(self, parsed_args):
+        client = self.app.client_manager.network
+        obj = client.find_network(parsed_args.network, ignore_missing=False)
+
+        # NOTE: As of now, UnsetNetwork has no attributes which need
+        # to be updated by update_network().
+
+        # tags is a subresource and it needs to be updated separately.
+        _tag.update_tags_for_unset(client, obj, parsed_args)
