@@ -25,7 +25,9 @@ from osc_lib import utils
 
 from openstackclient.i18n import _
 from openstackclient.identity import common as identity_common
+from openstackclient.network import common
 from openstackclient.network import sdk_utils
+from openstackclient.network.v2 import _tag
 
 
 LOG = logging.getLogger(__name__)
@@ -35,10 +37,6 @@ def _format_admin_state(state):
     return 'UP' if state else 'DOWN'
 
 
-def _format_dns_assignment(dns_assignment):
-    return utils.format_list_of_dicts(dns_assignment) \
-        if dns_assignment else None
-
 _formatters = {
     'admin_state_up': _format_admin_state,
     'is_admin_state_up': _format_admin_state,
@@ -47,10 +45,11 @@ _formatters = {
     'binding_vif_details': utils.format_dict,
     'binding:profile': utils.format_dict,
     'binding:vif_details': utils.format_dict,
-    'dns_assignment': _format_dns_assignment,
+    'dns_assignment': utils.format_list_of_dicts,
     'extra_dhcp_opts': utils.format_list_of_dicts,
     'fixed_ips': utils.format_list_of_dicts,
     'security_group_ids': utils.format_list,
+    'tags': utils.format_list,
 }
 
 
@@ -129,11 +128,13 @@ def _get_attrs(client_manager, parsed_args):
     if parsed_args.mac_address is not None:
         attrs['mac_address'] = parsed_args.mac_address
 
+    if parsed_args.dns_domain is not None:
+        attrs['dns_domain'] = parsed_args.dns_domain
     if parsed_args.dns_name is not None:
         attrs['dns_name'] = parsed_args.dns_name
     # It is possible that name is not updated during 'port set'
     if parsed_args.name is not None:
-        attrs['name'] = str(parsed_args.name)
+        attrs['name'] = parsed_args.name
     # The remaining options do not support 'port set' command, so they require
     # additional check
     if 'network' in parsed_args and parsed_args.network is not None:
@@ -162,6 +163,13 @@ def _get_attrs(client_manager, parsed_args):
         attrs['qos_policy_id'] = client_manager.network.find_qos_policy(
             parsed_args.qos_policy, ignore_missing=False).id
 
+    if ('enable_uplink_status_propagation' in parsed_args and
+            parsed_args.enable_uplink_status_propagation):
+        attrs['propagate_uplink_status'] = True
+    if ('disable_uplink_status_propagation' in parsed_args and
+            parsed_args.disable_uplink_status_propagation):
+        attrs['propagate_uplink_status'] = False
+
     return attrs
 
 
@@ -215,6 +223,9 @@ def _prepare_filter_fixed_ips(client_manager, parsed_args):
 
         if 'ip-address' in ip_spec:
             ips.append('ip_address=%s' % ip_spec['ip-address'])
+
+        if 'ip-substring' in ip_spec:
+            ips.append('ip_address_substr=%s' % ip_spec['ip-substring'])
     return ips
 
 
@@ -252,9 +263,10 @@ def _add_updatable_args(parser):
         '--vnic-type',
         metavar='<vnic-type>',
         choices=['direct', 'direct-physical', 'macvtap',
-                 'normal', 'baremetal'],
+                 'normal', 'baremetal', 'virtio-forwarder'],
         help=_("VNIC type for this port (direct | direct-physical | "
-               "macvtap | normal | baremetal, default: normal)")
+               "macvtap | normal | baremetal | virtio-forwarder, "
+               "default: normal)")
     )
     # NOTE(dtroyer): --host-id is deprecated in Mar 2016.  Do not
     #                remove before 3.x release or Mar 2017.
@@ -269,10 +281,16 @@ def _add_updatable_args(parser):
         metavar='<host-id>',
         help=argparse.SUPPRESS,
     )
+    parser.add_argument(
+        '--dns-domain',
+        metavar='dns-domain',
+        help=_("Set DNS domain to this port "
+               "(requires dns_domain extension for ports)")
+    )
     parser.add_argument(
         '--dns-name',
-        metavar='dns-name',
-        help=_("Set DNS name to this port "
+        metavar='<dns-name>',
+        help=_("Set DNS name for this port "
                "(requires DNS integration extension)")
     )
 
@@ -303,7 +321,8 @@ class CreatePort(command.ShowOne):
             help=_("Network this port belongs to (name or ID)")
         )
         _add_updatable_args(parser)
-        parser.add_argument(
+        fixed_ip = parser.add_mutually_exclusive_group()
+        fixed_ip.add_argument(
             '--fixed-ip',
             metavar='subnet=<subnet>,ip-address=<ip-address>',
             action=parseractions.MultiKeyValueAction,
@@ -312,6 +331,11 @@ class CreatePort(command.ShowOne):
                    "subnet=<subnet>,ip-address=<ip-address> "
                    "(repeat option to set multiple fixed IP addresses)")
         )
+        fixed_ip.add_argument(
+            '--no-fixed-ip',
+            action='store_true',
+            help=_("No IP or subnet for this port.")
+        )
         parser.add_argument(
             '--binding-profile',
             metavar='<binding-profile>',
@@ -332,6 +356,17 @@ class CreatePort(command.ShowOne):
             action='store_true',
             help=_("Disable port")
         )
+        uplink_status_group = parser.add_mutually_exclusive_group()
+        uplink_status_group.add_argument(
+            '--enable-uplink-status-propagation',
+            action='store_true',
+            help=_("Enable uplink status propagate")
+        )
+        uplink_status_group.add_argument(
+            '--disable-uplink-status-propagation',
+            action='store_true',
+            help=_("Disable uplink status propagate (default)")
+        )
         parser.add_argument(
             '--project',
             metavar='<project>',
@@ -387,6 +422,7 @@ class CreatePort(command.ShowOne):
                    "ip-address=<ip-address>[,mac-address=<mac-address>] "
                    "(repeat option to set multiple allowed-address pairs)")
         )
+        _tag.add_tag_option_to_parser_for_create(parser, _('port'))
         return parser
 
     def take_action(self, parsed_args):
@@ -402,6 +438,8 @@ class CreatePort(command.ShowOne):
 
         if parsed_args.fixed_ip:
             attrs['fixed_ips'] = parsed_args.fixed_ip
+        elif parsed_args.no_fixed_ip:
+            attrs['fixed_ips'] = []
 
         if parsed_args.security_group:
             attrs['security_group_ids'] = [client.find_security_group(
@@ -418,7 +456,12 @@ class CreatePort(command.ShowOne):
         if parsed_args.qos_policy:
             attrs['qos_policy_id'] = client.find_qos_policy(
                 parsed_args.qos_policy, ignore_missing=False).id
-        obj = client.create_port(**attrs)
+        with common.check_missing_extension_if_error(
+                self.app.client_manager.network, attrs):
+            obj = client.create_port(**attrs)
+
+        # tags cannot be set when created, so tags need to be set later.
+        _tag.update_tags_for_set(client, obj, parsed_args)
         display_columns, columns = _get_columns(obj)
         data = utils.get_item_properties(obj, columns, formatters=_formatters)
 
@@ -489,6 +532,11 @@ class ListPort(command.Lister):
             metavar='<server>',
             help=_("List only ports attached to this server (name or ID)"),
         )
+        device_group.add_argument(
+            '--device-id',
+            metavar='<device-id>',
+            help=_("List only ports with the specified device ID")
+        )
         parser.add_argument(
             '--mac-address',
             metavar='<mac-address>',
@@ -508,13 +556,16 @@ class ListPort(command.Lister):
         identity_common.add_project_domain_option_to_parser(parser)
         parser.add_argument(
             '--fixed-ip',
-            metavar='subnet=<subnet>,ip-address=<ip-address>',
+            metavar=('subnet=<subnet>,ip-address=<ip-address>,'
+                     'ip-substring=<ip-substring>'),
             action=parseractions.MultiKeyValueAction,
-            optional_keys=['subnet', 'ip-address'],
+            optional_keys=['subnet', 'ip-address', 'ip-substring'],
             help=_("Desired IP and/or subnet for filtering ports "
-                   "(name or ID): subnet=<subnet>,ip-address=<ip-address> "
+                   "(name or ID): subnet=<subnet>,ip-address=<ip-address>,"
+                   "ip-substring=<ip-substring> "
                    "(repeat option to set multiple fixed IP addresses)"),
         )
+        _tag.add_tag_filtering_option_to_parser(parser, _('ports'))
         return parser
 
     def take_action(self, parsed_args):
@@ -538,10 +589,12 @@ class ListPort(command.Lister):
 
         filters = {}
         if parsed_args.long:
-            columns += ('security_group_ids', 'device_owner',)
-            column_headers += ('Security Groups', 'Device Owner',)
+            columns += ('security_group_ids', 'device_owner', 'tags')
+            column_headers += ('Security Groups', 'Device Owner', 'Tags')
         if parsed_args.device_owner is not None:
             filters['device_owner'] = parsed_args.device_owner
+        if parsed_args.device_id is not None:
+            filters['device_id'] = parsed_args.device_id
         if parsed_args.router:
             _router = network_client.find_router(parsed_args.router,
                                                  ignore_missing=False)
@@ -569,11 +622,15 @@ class ListPort(command.Lister):
             filters['fixed_ips'] = _prepare_filter_fixed_ips(
                 self.app.client_manager, parsed_args)
 
+        _tag.get_tag_filtering_args(parsed_args, filters)
+
         data = network_client.ports(**filters)
 
-        return (column_headers,
+        headers, attrs = utils.calculate_header_and_attrs(
+            column_headers, columns, parsed_args)
+        return (headers,
                 (utils.get_item_properties(
-                    s, columns,
+                    s, attrs,
                     formatters=_formatters,
                 ) for s in data))
 
@@ -630,7 +687,7 @@ class SetPort(command.Command):
         parser.add_argument(
             '--no-binding-profile',
             action='store_true',
-            help=_("Clear existing information of binding:profile."
+            help=_("Clear existing information of binding:profile. "
                    "Specify both --binding-profile and --no-binding-profile "
                    "to overwrite the current binding:profile information.")
         )
@@ -684,11 +741,21 @@ class SetPort(command.Command):
             '--no-allowed-address',
             dest='no_allowed_address_pair',
             action='store_true',
-            help=_("Clear existing allowed-address pairs associated"
-                   "with this port."
-                   "(Specify both --allowed-address and --no-allowed-address"
+            help=_("Clear existing allowed-address pairs associated "
+                   "with this port. "
+                   "(Specify both --allowed-address and --no-allowed-address "
                    "to overwrite the current allowed-address pairs)")
         )
+        parser.add_argument(
+            '--data-plane-status',
+            metavar='<status>',
+            choices=['ACTIVE', 'DOWN'],
+            help=_("Set data plane status of this port (ACTIVE | DOWN). "
+                   "Unset it to None with the 'port unset' command "
+                   "(requires data plane status extension)")
+        )
+        _tag.add_tag_option_to_parser_for_set(parser, _('port'))
+
         return parser
 
     def take_action(self, parsed_args):
@@ -741,8 +808,16 @@ class SetPort(command.Command):
             attrs['allowed_address_pairs'].extend(
                 _convert_address_pairs(parsed_args)
             )
+        if parsed_args.data_plane_status:
+            attrs['data_plane_status'] = parsed_args.data_plane_status
 
-        client.update_port(obj, **attrs)
+        if attrs:
+            with common.check_missing_extension_if_error(
+                    self.app.client_manager.network, attrs):
+                client.update_port(obj, **attrs)
+
+        # tags is a subresource and it needs to be updated separately.
+        _tag.update_tags_for_set(client, obj, parsed_args)
 
 
 class ShowPort(command.ShowOne):
@@ -786,7 +861,7 @@ class UnsetPort(command.Command):
             '--binding-profile',
             metavar='<binding-profile-key>',
             action='append',
-            help=_("Desired key which should be removed from binding:profile"
+            help=_("Desired key which should be removed from binding:profile "
                    "(repeat option to unset multiple binding:profile data)"))
         parser.add_argument(
             '--security-group',
@@ -810,8 +885,8 @@ class UnsetPort(command.Command):
             required_keys=['ip-address'],
             optional_keys=['mac-address'],
             help=_("Desired allowed-address pair which should be removed "
-                   "from this port: ip-address=<ip-address> "
-                   "[,mac-address=<mac-address>] (repeat option to set "
+                   "from this port: ip-address=<ip-address>"
+                   "[,mac-address=<mac-address>] (repeat option to unset "
                    "multiple allowed-address pairs)")
         )
         parser.add_argument(
@@ -820,6 +895,13 @@ class UnsetPort(command.Command):
             default=False,
             help=_("Remove the QoS policy attached to the port")
         )
+        parser.add_argument(
+            '--data-plane-status',
+            action='store_true',
+            help=_("Clear existing information of data plane status")
+        )
+
+        _tag.add_tag_option_to_parser_for_unset(parser, _('port'))
 
         return parser
 
@@ -871,6 +953,11 @@ class UnsetPort(command.Command):
             attrs['allowed_address_pairs'] = tmp_addr_pairs
         if parsed_args.qos_policy:
             attrs['qos_policy_id'] = None
+        if parsed_args.data_plane_status:
+            attrs['data_plane_status'] = None
 
         if attrs:
             client.update_port(obj, **attrs)
+
+        # tags is a subresource and it needs to be updated separately.
+        _tag.update_tags_for_unset(client, obj, parsed_args)
