@@ -71,13 +71,21 @@ def _get_columns(item):
     }
     if hasattr(item, 'interfaces_info'):
         column_map['interfaces_info'] = 'interfaces_info'
-    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)
+    invisible_columns = []
+    if item.is_ha is None:
+        invisible_columns.append('is_ha')
+        column_map.pop('is_ha')
+    if item.is_distributed is None:
+        invisible_columns.append('is_distributed')
+        column_map.pop('is_distributed')
+    return sdk_utils.get_osc_show_columns_for_sdk_resource(
+        item, column_map, invisible_columns)
 
 
 def _get_attrs(client_manager, parsed_args):
     attrs = {}
     if parsed_args.name is not None:
-        attrs['name'] = str(parsed_args.name)
+        attrs['name'] = parsed_args.name
     if parsed_args.enable:
         attrs['admin_state_up'] = True
     if parsed_args.disable:
@@ -330,8 +338,6 @@ class ListRouter(command.Lister):
             'name',
             'status',
             'is_admin_state_up',
-            'is_distributed',
-            'is_ha',
             'project_id',
         )
         column_headers = (
@@ -339,8 +345,6 @@ class ListRouter(command.Lister):
             'Name',
             'Status',
             'State',
-            'Distributed',
-            'HA',
             'Project',
         )
 
@@ -376,6 +380,16 @@ class ListRouter(command.Lister):
         else:
             data = client.routers(**args)
 
+        # check if "HA" and "Distributed" columns should be displayed also
+        data = list(data)
+        for d in data:
+            if (d.is_distributed is not None and
+                    'is_distributed' not in columns):
+                columns = columns + ('is_distributed',)
+                column_headers = column_headers + ('Distributed',)
+            if d.is_ha is not None and 'is_ha' not in columns:
+                columns = columns + ('is_ha',)
+                column_headers = column_headers + ('HA',)
         if parsed_args.long:
             columns = columns + (
                 'routes',
@@ -563,7 +577,7 @@ class SetRouter(command.Command):
             metavar='subnet=<subnet>,ip-address=<ip-address>',
             action=parseractions.MultiKeyValueAction,
             optional_keys=['subnet', 'ip-address'],
-            help=_("Desired IP and/or subnet (name or ID)"
+            help=_("Desired IP and/or subnet (name or ID) "
                    "on external gateway: "
                    "subnet=<subnet>,ip-address=<ip-address> "
                    "(repeat option to set multiple fixed IP addresses)")
@@ -579,6 +593,17 @@ class SetRouter(command.Command):
             action='store_true',
             help=_("Disable Source NAT on external gateway")
         )
+        qos_policy_group = parser.add_mutually_exclusive_group()
+        qos_policy_group.add_argument(
+            '--qos-policy',
+            metavar='<qos-policy>',
+            help=_("Attach QoS policy to router gateway IPs")
+        )
+        qos_policy_group.add_argument(
+            '--no-qos-policy',
+            action='store_true',
+            help=_("Remove QoS policy from router gateway IPs")
+        )
         _tag.add_tag_option_to_parser_for_set(parser, _('router'))
         return parser
 
@@ -611,10 +636,10 @@ class SetRouter(command.Command):
         elif parsed_args.no_route or parsed_args.clear_routes:
             attrs['routes'] = []
         if (parsed_args.disable_snat or parsed_args.enable_snat or
-           parsed_args.fixed_ip) and not parsed_args.external_gateway:
-                msg = (_("You must specify '--external-gateway' in order"
-                         "to update the SNAT or fixed-ip values"))
-                raise exceptions.CommandError(msg)
+                parsed_args.fixed_ip) and not parsed_args.external_gateway:
+            msg = (_("You must specify '--external-gateway' in order "
+                     "to update the SNAT or fixed-ip values"))
+            raise exceptions.CommandError(msg)
         if parsed_args.external_gateway:
             gateway_info = {}
             network = client.find_network(
@@ -638,6 +663,27 @@ class SetRouter(command.Command):
                     ips.append(ip_spec)
                 gateway_info['external_fixed_ips'] = ips
             attrs['external_gateway_info'] = gateway_info
+
+        if ((parsed_args.qos_policy or parsed_args.no_qos_policy) and
+                not parsed_args.external_gateway):
+            try:
+                original_net_id = obj.external_gateway_info['network_id']
+            except (KeyError, TypeError):
+                msg = (_("You must specify '--external-gateway' or the router "
+                         "must already have an external network in order to "
+                         "set router gateway IP QoS"))
+                raise exceptions.CommandError(msg)
+            else:
+                if not attrs.get('external_gateway_info'):
+                    attrs['external_gateway_info'] = {}
+                attrs['external_gateway_info']['network_id'] = original_net_id
+        if parsed_args.qos_policy:
+            check_qos_id = client.find_qos_policy(
+                parsed_args.qos_policy, ignore_missing=False).id
+            attrs['external_gateway_info']['qos_policy_id'] = check_qos_id
+
+        if 'no_qos_policy' in parsed_args and parsed_args.no_qos_policy:
+            attrs['external_gateway_info']['qos_policy_id'] = None
         if attrs:
             client.update_router(obj, **attrs)
         # tags is a subresource and it needs to be updated separately.
@@ -701,6 +747,12 @@ class UnsetRouter(command.Command):
             action='store_true',
             default=False,
             help=_("Remove external gateway information from the router"))
+        parser.add_argument(
+            '--qos-policy',
+            action='store_true',
+            default=False,
+            help=_("Remove QoS policy from router gateway IPs")
+        )
         parser.add_argument(
             'router',
             metavar="<router>",
@@ -713,6 +765,7 @@ class UnsetRouter(command.Command):
         client = self.app.client_manager.network
         obj = client.find_router(parsed_args.router, ignore_missing=False)
         tmp_routes = copy.deepcopy(obj.routes)
+        tmp_external_gateway_info = copy.deepcopy(obj.external_gateway_info)
         attrs = {}
         if parsed_args.routes:
             try:
@@ -723,6 +776,20 @@ class UnsetRouter(command.Command):
                 msg = (_("Router does not contain route %s") % route)
                 raise exceptions.CommandError(msg)
             attrs['routes'] = tmp_routes
+        if parsed_args.qos_policy:
+            try:
+                if (tmp_external_gateway_info['network_id'] and
+                        tmp_external_gateway_info['qos_policy_id']):
+                    pass
+            except (KeyError, TypeError):
+                msg = _("Router does not have external network or qos policy")
+                raise exceptions.CommandError(msg)
+            else:
+                attrs['external_gateway_info'] = {
+                    'network_id': tmp_external_gateway_info['network_id'],
+                    'qos_policy_id': None
+                }
+
         if parsed_args.external_gateway:
             attrs['external_gateway_info'] = {}
         if attrs:
