@@ -135,7 +135,7 @@ def _get_columns(item):
         'tenant_id': 'project_id',
     }
     # Do not show this column when displaying a subnet
-    invisible_columns = ['use_default_subnetpool']
+    invisible_columns = ['use_default_subnet_pool']
     return sdk_utils.get_osc_show_columns_for_sdk_resource(
         item,
         column_map,
@@ -167,8 +167,9 @@ def convert_entries_to_gateway(entries):
 
 def _get_attrs(client_manager, parsed_args, is_create=True):
     attrs = {}
+    client = client_manager.network
     if 'name' in parsed_args and parsed_args.name is not None:
-        attrs['name'] = str(parsed_args.name)
+        attrs['name'] = parsed_args.name
 
     if is_create:
         if 'project' in parsed_args and parsed_args.project is not None:
@@ -179,13 +180,14 @@ def _get_attrs(client_manager, parsed_args, is_create=True):
                 parsed_args.project_domain,
             ).id
             attrs['tenant_id'] = project_id
-        client = client_manager.network
         attrs['network_id'] = client.find_network(parsed_args.network,
                                                   ignore_missing=False).id
         if parsed_args.subnet_pool is not None:
             subnet_pool = client.find_subnet_pool(parsed_args.subnet_pool,
                                                   ignore_missing=False)
             attrs['subnetpool_id'] = subnet_pool.id
+        if parsed_args.use_prefix_delegation:
+            attrs['subnetpool_id'] = "prefix_delegation"
         if parsed_args.use_default_subnet_pool:
             attrs['use_default_subnet_pool'] = True
         if parsed_args.prefix_length is not None:
@@ -198,10 +200,10 @@ def _get_attrs(client_manager, parsed_args, is_create=True):
             attrs['ipv6_ra_mode'] = parsed_args.ipv6_ra_mode
         if parsed_args.ipv6_address_mode is not None:
             attrs['ipv6_address_mode'] = parsed_args.ipv6_address_mode
-        if parsed_args.network_segment is not None:
-            attrs['segment_id'] = client.find_segment(
-                parsed_args.network_segment, ignore_missing=False).id
 
+    if parsed_args.network_segment is not None:
+        attrs['segment_id'] = client.find_segment(
+            parsed_args.network_segment, ignore_missing=False).id
     if 'gateway' in parsed_args and parsed_args.gateway is not None:
         gateway = parsed_args.gateway.lower()
 
@@ -245,6 +247,7 @@ class CreateSubnet(command.ShowOne):
         parser = super(CreateSubnet, self).get_parser(prog_name)
         parser.add_argument(
             'name',
+            metavar='<name>',
             help=_("New subnet name")
         )
         parser.add_argument(
@@ -260,6 +263,11 @@ class CreateSubnet(command.ShowOne):
             help=_("Subnet pool from which this subnet will obtain a CIDR "
                    "(Name or ID)")
         )
+        subnet_pool_group.add_argument(
+            '--use-prefix-delegation',
+            help=_("Use 'prefix-delegation' if IP is IPv6 format "
+                   "and IP would be delegated externally")
+        )
         subnet_pool_group.add_argument(
             '--use-default-subnet-pool',
             action='store_true',
@@ -405,7 +413,7 @@ class ListSubnet(command.Lister):
             choices=[4, 6],
             metavar='<ip-version>',
             dest='ip_version',
-            help=_("List only subnets of given IP version in output."
+            help=_("List only subnets of given IP version in output. "
                    "Allowed values for IP version are 4 and 6."),
         )
         dhcp_enable_group = parser.add_mutually_exclusive_group()
@@ -551,6 +559,14 @@ class SetSubnet(command.Command):
                    "'none': This subnet will not use a gateway, "
                    "e.g.: --gateway 192.168.9.1, --gateway none.")
         )
+        parser.add_argument(
+            '--network-segment',
+            metavar='<network-segment>',
+            help=_("Network segment to associate with this subnet (name or "
+                   "ID). It is only allowed to set the segment if the current "
+                   "value is `None`, the network must also have only one "
+                   "segment and only one subnet can exist on the network.")
+        )
         parser.add_argument(
             '--description',
             metavar='<description>',
@@ -574,7 +590,7 @@ class SetSubnet(command.Command):
             if not parsed_args.no_host_route:
                 attrs['host_routes'] += obj.host_routes
         elif parsed_args.no_host_route:
-            attrs['host_routes'] = ''
+            attrs['host_routes'] = []
         if 'allocation_pools' in attrs:
             if not parsed_args.no_allocation_pool:
                 attrs['allocation_pools'] += obj.allocation_pools
