@@ -13,14 +13,22 @@
 import json
 import uuid
 
-from openstackclient.tests.functional import base
+from openstackclient.tests.functional.network.v2 import common
 
 
-class RouterTests(base.TestCase):
-    """Functional tests for router. """
+class RouterTests(common.NetworkTagTests):
+    """Functional tests for router"""
+
+    base_command = 'router'
+
+    def setUp(self):
+        super(RouterTests, self).setUp()
+        # Nothing in this class works with Nova Network
+        if not self.haz_network:
+            self.skipTest("No Network service present")
 
     def test_router_create_and_delete(self):
-        """Test create options, delete"""
+        """Test create options, delete multiple"""
         name1 = uuid.uuid4().hex
         name2 = uuid.uuid4().hex
         cmd_output = json.loads(self.openstack(
@@ -75,6 +83,8 @@ class RouterTests(base.TestCase):
             '--disable ' +
             name1
         ))
+
+        self.addCleanup(self.openstack, 'router delete ' + name1)
         self.assertEqual(
             name1,
             cmd_output["name"],
@@ -92,6 +102,8 @@ class RouterTests(base.TestCase):
             '--project ' + demo_project_id +
             ' ' + name2
         ))
+
+        self.addCleanup(self.openstack, 'router delete ' + name2)
         self.assertEqual(
             name2,
             cmd_output["name"],
@@ -141,12 +153,42 @@ class RouterTests(base.TestCase):
         self.assertIn(name1, names)
         self.assertIn(name2, names)
 
-        del_output = self.openstack(
-            'router delete ' + name1 + ' ' + name2)
-        self.assertOutput('', del_output)
+    def test_router_list_l3_agent(self):
+        """Tests create router, add l3 agent, list, delete"""
+        name = uuid.uuid4().hex
+        cmd_output = json.loads(self.openstack(
+            'router create -f json ' + name))
+
+        self.addCleanup(self.openstack, 'router delete ' + name)
+        # Get router ID
+        router_id = cmd_output['id']
+        # Get l3 agent id
+        cmd_output = json.loads(self.openstack(
+            'network agent list -f json --agent-type l3'))
+
+        # Check at least one L3 agent is included in the response.
+        self.assertTrue(cmd_output)
+        agent_id = cmd_output[0]['ID']
+
+        # Add router to agent
+        self.openstack(
+            'network agent add router --l3 ' + agent_id + ' ' + router_id)
+
+        cmd_output = json.loads(self.openstack(
+            'router list -f json --agent ' + agent_id))
+        router_ids = [x['ID'] for x in cmd_output]
+        self.assertIn(router_id, router_ids)
+
+        # Remove router from agent
+        self.openstack(
+            'network agent remove router --l3 ' + agent_id + ' ' + router_id)
+        cmd_output = json.loads(self.openstack(
+            'router list -f json --agent ' + agent_id))
+        router_ids = [x['ID'] for x in cmd_output]
+        self.assertNotIn(router_id, router_ids)
 
     def test_router_set_show_unset(self):
-        """Tests create router, set, unset, show, delete"""
+        """Tests create router, set, unset, show"""
 
         name = uuid.uuid4().hex
         new_name = name + "_"
@@ -155,6 +197,7 @@ class RouterTests(base.TestCase):
             '--description aaaa ' +
             name
         ))
+        self.addCleanup(self.openstack, 'router delete ' + new_name)
         self.assertEqual(
             name,
             cmd_output["name"],
@@ -204,10 +247,7 @@ class RouterTests(base.TestCase):
             'router show -f json ' +
             new_name
         ))
-        self.assertEqual(
-            True,
-            cmd_output["distributed"],
-        )
+        self.assertTrue(cmd_output["distributed"])
         self.assertIsNotNone(cmd_output["external_gateway_info"])
 
         # Test unset
@@ -221,7 +261,3 @@ class RouterTests(base.TestCase):
             new_name
         ))
         self.assertIsNone(cmd_output["external_gateway_info"])
-
-        del_output = self.openstack(
-            'router delete ' + new_name)
-        self.assertOutput('', del_output)
