@@ -15,6 +15,7 @@
 
 import copy
 
+from glanceclient.common import utils as glanceclient_utils
 from glanceclient.v2 import schemas
 import mock
 from osc_lib import exceptions
@@ -526,6 +527,7 @@ class TestImageList(TestImage):
         verifylist = [
             ('public', False),
             ('private', False),
+            ('community', False),
             ('shared', False),
             ('long', False),
         ]
@@ -549,6 +551,7 @@ class TestImageList(TestImage):
         verifylist = [
             ('public', True),
             ('private', False),
+            ('community', False),
             ('shared', False),
             ('long', False),
         ]
@@ -573,6 +576,7 @@ class TestImageList(TestImage):
         verifylist = [
             ('public', False),
             ('private', True),
+            ('community', False),
             ('shared', False),
             ('long', False),
         ]
@@ -590,6 +594,31 @@ class TestImageList(TestImage):
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.datalist, tuple(data))
 
+    def test_image_list_community_option(self):
+        arglist = [
+            '--community',
+        ]
+        verifylist = [
+            ('public', False),
+            ('private', False),
+            ('community', True),
+            ('shared', False),
+            ('long', False),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        # In base command class Lister in cliff, abstract method take_action()
+        # returns a tuple containing the column names and an iterable
+        # containing the data to be listed.
+        columns, data = self.cmd.take_action(parsed_args)
+        self.api_mock.image_list.assert_called_with(
+            community=True,
+            marker=self._image.id,
+        )
+
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.datalist, tuple(data))
+
     def test_image_list_shared_option(self):
         arglist = [
             '--shared',
@@ -597,6 +626,7 @@ class TestImageList(TestImage):
         verifylist = [
             ('public', False),
             ('private', False),
+            ('community', False),
             ('shared', True),
             ('long', False),
         ]
@@ -614,6 +644,49 @@ class TestImageList(TestImage):
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.datalist, tuple(data))
 
+    def test_image_list_shared_member_status_option(self):
+        arglist = [
+            '--shared',
+            '--member-status', 'all'
+        ]
+        verifylist = [
+            ('public', False),
+            ('private', False),
+            ('community', False),
+            ('shared', True),
+            ('long', False),
+            ('member_status', 'all')
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        # In base command class Lister in cliff, abstract method take_action()
+        # returns a tuple containing the column names and an iterable
+        # containing the data to be listed.
+        columns, data = self.cmd.take_action(parsed_args)
+        self.api_mock.image_list.assert_called_with(
+            shared=True,
+            member_status='all',
+            marker=self._image.id,
+        )
+
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.datalist, tuple(data))
+
+    def test_image_list_shared_member_status_lower(self):
+        arglist = [
+            '--shared',
+            '--member-status', 'ALl'
+        ]
+        verifylist = [
+            ('public', False),
+            ('private', False),
+            ('community', False),
+            ('shared', True),
+            ('long', False),
+            ('member_status', 'all')
+        ]
+        self.check_parser(self.cmd, arglist, verifylist)
+
     def test_image_list_long_option(self):
         arglist = [
             '--long',
@@ -707,7 +780,8 @@ class TestImageList(TestImage):
         )
         si_mock.assert_called_with(
             [self._image],
-            'name:asc'
+            'name:asc',
+            str,
         )
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.datalist, tuple(data))
@@ -778,6 +852,66 @@ class TestImageList(TestImage):
             status='active', marker=self._image.id
         )
 
+    def test_image_list_tag_option(self):
+        arglist = [
+            '--tag', 'abc',
+        ]
+        verifylist = [
+            ('tag', 'abc'),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        columns, data = self.cmd.take_action(parsed_args)
+        self.api_mock.image_list.assert_called_with(
+            tag='abc', marker=self._image.id
+        )
+
+
+class TestListImageProjects(TestImage):
+
+    project = identity_fakes.FakeProject.create_one_project()
+    _image = image_fakes.FakeImage.create_one_image()
+    member = image_fakes.FakeImage.create_one_image_member(
+        attrs={'image_id': _image.id,
+               'member_id': project.id}
+    )
+
+    columns = (
+        "Image ID",
+        "Member ID",
+        "Status"
+    )
+
+    datalist = ((
+        _image.id,
+        member.member_id,
+        member.status,
+    ))
+
+    def setUp(self):
+        super(TestListImageProjects, self).setUp()
+
+        self.images_mock.get.return_value = self._image
+        self.image_members_mock.list.return_value = self.datalist
+
+        self.cmd = image.ListImageProjects(self.app, None)
+
+    def test_image_member_list(self):
+        arglist = [
+            self._image.id
+        ]
+        verifylist = [
+            ('image', self._image.id)
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        columns, data = self.cmd.take_action(parsed_args)
+
+        self.image_members_mock.list.assert_called_with(self._image.id)
+
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(len(self.datalist), len(tuple(data)))
+
 
 class TestRemoveProjectImage(TestImage):
 
@@ -1505,3 +1639,53 @@ class TestImageUnset(TestImage):
             self.image.id, 'test'
         )
         self.assertIsNone(result)
+
+
+class TestImageSave(TestImage):
+
+    image = image_fakes.FakeImage.create_one_image({})
+
+    def setUp(self):
+        super(TestImageSave, self).setUp()
+
+        # Generate a request id
+        self.resp = mock.MagicMock()
+        self.resp.headers['x-openstack-request-id'] = 'req_id'
+
+        # Get the command object to test
+        self.cmd = image.SaveImage(self.app, None)
+
+    def test_save_data(self):
+        req_id_proxy = glanceclient_utils.RequestIdProxy(
+            ['some_data', self.resp]
+        )
+        self.images_mock.data.return_value = req_id_proxy
+
+        arglist = ['--file', '/path/to/file', self.image.id]
+
+        verifylist = [
+            ('file', '/path/to/file'),
+            ('image', self.image.id)
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        with mock.patch('glanceclient.common.utils.save_image') as mocked_save:
+            self.cmd.take_action(parsed_args)
+            mocked_save.assert_called_once_with(req_id_proxy, '/path/to/file')
+
+    def test_save_no_data(self):
+        req_id_proxy = glanceclient_utils.RequestIdProxy(
+            [None, self.resp]
+        )
+        self.images_mock.data.return_value = req_id_proxy
+
+        arglist = ['--file', '/path/to/file', self.image.id]
+
+        verifylist = [
+            ('file', '/path/to/file'),
+            ('image', self.image.id)
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        # Raise SystemExit if no data was provided.
+        self.assertRaises(SystemExit, self.cmd.take_action, parsed_args)
