@@ -335,6 +335,7 @@ class FakeNetwork(object):
             'name': 'network-name-' + uuid.uuid4().hex,
             'status': 'ACTIVE',
             'description': 'network-description-' + uuid.uuid4().hex,
+            'dns_domain': 'example.org.',
             'mtu': '1350',
             'tenant_id': 'project-id-' + uuid.uuid4().hex,
             'admin_state_up': True,
@@ -537,6 +538,66 @@ class FakeNetworkSegment(object):
         return network_segments
 
 
+class FakeNetworkSegmentRange(object):
+    """Fake one or more network segment ranges."""
+
+    @staticmethod
+    def create_one_network_segment_range(attrs=None):
+        """Create a fake network segment range.
+
+        :param Dictionary attrs:
+            A dictionary with all attributes
+        :return:
+            A FakeResource object faking the network segment range
+        """
+        attrs = attrs or {}
+
+        # Set default attributes.
+        fake_uuid = uuid.uuid4().hex
+        network_segment_range_attrs = {
+            'id': 'network-segment-range-id-' + fake_uuid,
+            'name': 'network-segment-name-' + fake_uuid,
+            'default': False,
+            'shared': False,
+            'project_id': 'project-id-' + fake_uuid,
+            'network_type': 'vlan',
+            'physical_network': 'physical-network-name-' + fake_uuid,
+            'minimum': 100,
+            'maximum': 106,
+            'used': {104: '3312e4ba67864b2eb53f3f41432f8efc',
+                     106: '3312e4ba67864b2eb53f3f41432f8efc'},
+            'available': [100, 101, 102, 103, 105],
+        }
+
+        # Overwrite default attributes.
+        network_segment_range_attrs.update(attrs)
+
+        network_segment_range = fakes.FakeResource(
+            info=copy.deepcopy(network_segment_range_attrs),
+            loaded=True
+        )
+
+        return network_segment_range
+
+    @staticmethod
+    def create_network_segment_ranges(attrs=None, count=2):
+        """Create multiple fake network segment ranges.
+
+        :param Dictionary attrs:
+            A dictionary with all attributes
+        :param int count:
+            The number of network segment ranges to fake
+        :return:
+            A list of FakeResource objects faking the network segment ranges
+        """
+        network_segment_ranges = []
+        for i in range(0, count):
+            network_segment_ranges.append(
+                FakeNetworkSegmentRange.create_one_network_segment_range(attrs)
+            )
+        return network_segment_ranges
+
+
 class FakePort(object):
     """Fake one or more ports."""
 
@@ -580,6 +641,7 @@ class FakePort(object):
             'tenant_id': 'project-id-' + uuid.uuid4().hex,
             'qos_policy_id': 'qos-policy-id-' + uuid.uuid4().hex,
             'tags': [],
+            'uplink_status_propagation': False,
         }
 
         # Overwrite default attributes.
@@ -599,6 +661,8 @@ class FakePort(object):
         port.project_id = port_attrs['tenant_id']
         port.security_group_ids = port_attrs['security_group_ids']
         port.qos_policy_id = port_attrs['qos_policy_id']
+        port.uplink_status_propagation = port_attrs[
+            'uplink_status_propagation']
 
         return port
 
@@ -1696,3 +1760,26 @@ class FakeQuota(object):
             info=copy.deepcopy(quota_attrs),
             loaded=True)
         return quota
+
+    @staticmethod
+    def create_one_net_detailed_quota(attrs=None):
+        """Create one quota"""
+        attrs = attrs or {}
+
+        quota_attrs = {
+            'floating_ips': {'used': 0, 'reserved': 0, 'limit': 20},
+            'networks': {'used': 0, 'reserved': 0, 'limit': 25},
+            'ports': {'used': 0, 'reserved': 0, 'limit': 11},
+            'rbac_policies': {'used': 0, 'reserved': 0, 'limit': 15},
+            'routers': {'used': 0, 'reserved': 0, 'limit': 40},
+            'security_groups': {'used': 0, 'reserved': 0, 'limit': 10},
+            'security_group_rules': {'used': 0, 'reserved': 0, 'limit': 100},
+            'subnets': {'used': 0, 'reserved': 0, 'limit': 20},
+            'subnet_pools': {'used': 0, 'reserved': 0, 'limit': 30}}
+
+        quota_attrs.update(attrs)
+
+        quota = fakes.FakeResource(
+            info=copy.deepcopy(quota_attrs),
+            loaded=True)
+        return quota
