@@ -137,7 +137,7 @@ class TestCreateRouter(TestRouter):
         osc_utils.format_list(new_router.availability_zones),
         new_router.description,
         new_router.distributed,
-        router._format_external_gateway_info(new_router.external_gateway_info),
+        router._format_router_info(new_router.external_gateway_info),
         new_router.ha,
         new_router.id,
         new_router.name,
@@ -400,9 +400,9 @@ class TestListRouter(TestRouter):
         'Name',
         'Status',
         'State',
+        'Project',
         'Distributed',
         'HA',
-        'Project',
     )
     columns_long = columns + (
         'Routes',
@@ -423,9 +423,9 @@ class TestListRouter(TestRouter):
             r.name,
             r.status,
             router._format_admin_state(r.admin_state_up),
+            r.tenant_id,
             r.distributed,
             r.ha,
-            r.tenant_id,
         ))
 
     router_agent_data = []
@@ -448,7 +448,7 @@ class TestListRouter(TestRouter):
         data_long.append(
             data[i] + (
                 router._format_routes(r.routes),
-                router._format_external_gateway_info(r.external_gateway_info),
+                router._format_router_info(r.external_gateway_info),
                 osc_utils.format_list(r.availability_zones),
                 osc_utils.format_list(r.tags),
             )
@@ -459,7 +459,7 @@ class TestListRouter(TestRouter):
         data_long_no_az.append(
             data[i] + (
                 router._format_routes(r.routes),
-                router._format_external_gateway_info(r.external_gateway_info),
+                router._format_router_info(r.external_gateway_info),
                 osc_utils.format_list(r.tags),
             )
         )
@@ -496,6 +496,25 @@ class TestListRouter(TestRouter):
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.data, list(data))
 
+    def test_router_list_no_ha_no_distributed(self):
+        _routers = network_fakes.FakeRouter.create_routers({
+            'ha': None,
+            'distributed': None},
+            count=3)
+
+        arglist = []
+        verifylist = [
+            ('long', False),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        with mock.patch.object(
+                self.network, "routers", return_value=_routers):
+            columns, data = self.cmd.take_action(parsed_args)
+
+        self.assertNotIn("is_distributed", columns)
+        self.assertNotIn("is_ha", columns)
+
     def test_router_list_long(self):
         arglist = [
             '--long',
@@ -1113,11 +1132,116 @@ class TestSetRouter(TestRouter):
     def test_set_with_no_tag(self):
         self._test_set_tags(with_tags=False)
 
+    def test_set_gateway_ip_qos(self):
+        qos_policy = network_fakes.FakeNetworkQosPolicy.create_one_qos_policy()
+        self.network.find_qos_policy = mock.Mock(return_value=qos_policy)
+        arglist = [
+            "--external-gateway", self._network.id,
+            "--qos-policy", qos_policy.id,
+            self._router.id,
+        ]
+        verifylist = [
+            ('router', self._router.id),
+            ('external_gateway', self._network.id),
+            ('qos_policy', qos_policy.id),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+        self.network.update_router.assert_called_with(
+            self._router, **{'external_gateway_info': {
+                'network_id': self._network.id,
+                'qos_policy_id': qos_policy.id, }})
+        self.assertIsNone(result)
+
+    def test_unset_gateway_ip_qos(self):
+        arglist = [
+            "--external-gateway", self._network.id,
+            "--no-qos-policy",
+            self._router.id,
+        ]
+        verifylist = [
+            ('router', self._router.id),
+            ('external_gateway', self._network.id),
+            ('no_qos_policy', True),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+        self.network.update_router.assert_called_with(
+            self._router, **{'external_gateway_info': {
+                'network_id': self._network.id,
+                'qos_policy_id': None, }})
+        self.assertIsNone(result)
+
+    def test_set_unset_gateway_ip_qos(self):
+        qos_policy = network_fakes.FakeNetworkQosPolicy.create_one_qos_policy()
+        self.network.find_qos_policy = mock.Mock(return_value=qos_policy)
+        arglist = [
+            "--external-gateway", self._network.id,
+            "--qos-policy", qos_policy.id,
+            "--no-qos-policy",
+            self._router.id,
+        ]
+        verifylist = [
+            ('router', self._router.id),
+            ('external_gateway', self._network.id),
+            ('qos_policy', qos_policy.id),
+            ('no_qos_policy', True),
+        ]
+
+        self.assertRaises(tests_utils.ParserException, self.check_parser,
+                          self.cmd, arglist, verifylist)
+
+    def test_set_gateway_ip_qos_no_gateway(self):
+        qos_policy = network_fakes.FakeNetworkQosPolicy.create_one_qos_policy()
+        self.network.find_qos_policy = mock.Mock(return_value=qos_policy)
+        router = network_fakes.FakeRouter.create_one_router()
+        self.network.find_router = mock.Mock(return_value=router)
+        arglist = [
+            "--qos-policy", qos_policy.id,
+            router.id,
+        ]
+        verifylist = [
+            ('router', router.id),
+            ('qos_policy', qos_policy.id),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        self.assertRaises(exceptions.CommandError,
+                          self.cmd.take_action, parsed_args)
+
+    def test_unset_gateway_ip_qos_no_gateway(self):
+        qos_policy = network_fakes.FakeNetworkQosPolicy.create_one_qos_policy()
+        self.network.find_qos_policy = mock.Mock(return_value=qos_policy)
+        router = network_fakes.FakeRouter.create_one_router()
+        self.network.find_router = mock.Mock(return_value=router)
+        arglist = [
+            "--no-qos-policy",
+            router.id,
+        ]
+        verifylist = [
+            ('router', router.id),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        self.assertRaises(exceptions.CommandError,
+                          self.cmd.take_action, parsed_args)
+
 
 class TestShowRouter(TestRouter):
 
     # The router to set.
     _router = network_fakes.FakeRouter.create_one_router()
+    _port = network_fakes.FakePort.create_one_port({
+        'device_owner': 'network:router_interface',
+        'device_id': _router.id
+    })
+    setattr(_router,
+            'interfaces_info',
+            [{'port_id': _port.id,
+              'ip_address': _port.fixed_ips[0]['ip_address'],
+              'subnet_id': _port.fixed_ips[0]['subnet_id']}])
 
     columns = (
         'admin_state_up',
@@ -1128,6 +1252,7 @@ class TestShowRouter(TestRouter):
         'external_gateway_info',
         'ha',
         'id',
+        'interfaces_info',
         'name',
         'project_id',
         'routes',
@@ -1140,9 +1265,10 @@ class TestShowRouter(TestRouter):
         osc_utils.format_list(_router.availability_zones),
         _router.description,
         _router.distributed,
-        router._format_external_gateway_info(_router.external_gateway_info),
+        router._format_router_info(_router.external_gateway_info),
         _router.ha,
         _router.id,
+        router._format_router_info(_router.interfaces_info),
         _router.name,
         _router.tenant_id,
         router._format_routes(_router.routes),
@@ -1154,6 +1280,7 @@ class TestShowRouter(TestRouter):
         super(TestShowRouter, self).setUp()
 
         self.network.find_router = mock.Mock(return_value=self._router)
+        self.network.ports = mock.Mock(return_value=[self._port])
 
         # Get the command object to test
         self.cmd = router.ShowRouter(self.app, self.namespace)
@@ -1178,20 +1305,50 @@ class TestShowRouter(TestRouter):
 
         self.network.find_router.assert_called_once_with(
             self._router.name, ignore_missing=False)
+        self.network.ports.assert_called_with(**{
+            'device_id': self._router.id
+        })
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.data, data)
 
+    def test_show_no_ha_no_distributed(self):
+        _router = network_fakes.FakeRouter.create_one_router({
+            'ha': None,
+            'distributed': None})
+
+        arglist = [
+            _router.name,
+        ]
+        verifylist = [
+            ('router', _router.name),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        with mock.patch.object(
+                self.network, "find_router", return_value=_router):
+            columns, data = self.cmd.take_action(parsed_args)
+
+        self.assertNotIn("is_distributed", columns)
+        self.assertNotIn("is_ha", columns)
+
 
 class TestUnsetRouter(TestRouter):
 
     def setUp(self):
         super(TestUnsetRouter, self).setUp()
+        self.fake_network = network_fakes.FakeNetwork.create_one_network()
+        self.fake_qos_policy = (
+            network_fakes.FakeNetworkQosPolicy.create_one_qos_policy())
         self._testrouter = network_fakes.FakeRouter.create_one_router(
             {'routes': [{"destination": "192.168.101.1/24",
                          "nexthop": "172.24.4.3"},
                         {"destination": "192.168.101.2/24",
                          "nexthop": "172.24.4.3"}],
-             'tags': ['green', 'red'], })
+             'tags': ['green', 'red'],
+             'external_gateway_info': {
+                 'network_id': self.fake_network.id,
+                 'qos_policy_id': self.fake_qos_policy.id
+            }})
         self.fake_subnet = network_fakes.FakeSubnet.create_one_subnet()
         self.network.find_router = mock.Mock(return_value=self._testrouter)
         self.network.update_router = mock.Mock(return_value=None)
@@ -1274,3 +1431,54 @@ class TestUnsetRouter(TestRouter):
 
     def test_unset_with_all_tag(self):
         self._test_unset_tags(with_tags=False)
+
+    def test_unset_router_qos_policy(self):
+        arglist = [
+            '--qos-policy',
+            self._testrouter.name,
+        ]
+        verifylist = [
+            ('qos_policy', True)
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        result = self.cmd.take_action(parsed_args)
+        attrs = {'external_gateway_info': {"network_id": self.fake_network.id,
+                                           "qos_policy_id": None}}
+        self.network.update_router.assert_called_once_with(
+            self._testrouter, **attrs)
+        self.assertIsNone(result)
+
+    def test_unset_gateway_ip_qos_no_network(self):
+        qos_policy = network_fakes.FakeNetworkQosPolicy.create_one_qos_policy()
+        self.network.find_qos_policy = mock.Mock(return_value=qos_policy)
+        router = network_fakes.FakeRouter.create_one_router()
+        self.network.find_router = mock.Mock(return_value=router)
+        arglist = [
+            "--qos-policy",
+            router.id,
+        ]
+        verifylist = [
+            ('router', router.id),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        self.assertRaises(exceptions.CommandError,
+                          self.cmd.take_action, parsed_args)
+
+    def test_unset_gateway_ip_qos_no_qos(self):
+        qos_policy = network_fakes.FakeNetworkQosPolicy.create_one_qos_policy()
+        self.network.find_qos_policy = mock.Mock(return_value=qos_policy)
+        router = network_fakes.FakeRouter.create_one_router(
+            {"external_gateway_info": {"network_id": "fake-id"}})
+        self.network.find_router = mock.Mock(return_value=router)
+        arglist = [
+            "--qos-policy",
+            router.id,
+        ]
+        verifylist = [
+            ('router', router.id),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        self.assertRaises(exceptions.CommandError,
+                          self.cmd.take_action, parsed_args)
