@@ -290,47 +290,47 @@ class FakePool(object):
 class FakeVolumeClient(object):
 
     def __init__(self, **kwargs):
-        self.volumes = mock.Mock()
-        self.volumes.resource_class = fakes.FakeResource(None, {})
+        self.auth_token = kwargs['token']
+        self.management_url = kwargs['endpoint']
+        self.availability_zones = mock.Mock()
+        self.availability_zones.resource_class = fakes.FakeResource(None, {})
+        self.backups = mock.Mock()
+        self.backups.resource_class = fakes.FakeResource(None, {})
+        self.capabilities = mock.Mock()
+        self.capabilities.resource_class = fakes.FakeResource(None, {})
+        self.cgsnapshots = mock.Mock()
+        self.cgsnapshots.resource_class = fakes.FakeResource(None, {})
+        self.consistencygroups = mock.Mock()
+        self.consistencygroups.resource_class = fakes.FakeResource(None, {})
         self.extensions = mock.Mock()
         self.extensions.resource_class = fakes.FakeResource(None, {})
         self.limits = mock.Mock()
         self.limits.resource_class = fakes.FakeResource(None, {})
-        self.volume_snapshots = mock.Mock()
-        self.volume_snapshots.resource_class = fakes.FakeResource(None, {})
-        self.backups = mock.Mock()
-        self.backups.resource_class = fakes.FakeResource(None, {})
-        self.volume_types = mock.Mock()
-        self.volume_types.resource_class = fakes.FakeResource(None, {})
-        self.volume_type_access = mock.Mock()
-        self.volume_type_access.resource_class = fakes.FakeResource(None, {})
+        self.pools = mock.Mock()
+        self.pools.resource_class = fakes.FakeResource(None, {})
+        self.qos_specs = mock.Mock()
+        self.qos_specs.resource_class = fakes.FakeResource(None, {})
+        self.quota_classes = mock.Mock()
+        self.quota_classes.resource_class = fakes.FakeResource(None, {})
+        self.quotas = mock.Mock()
+        self.quotas.resource_class = fakes.FakeResource(None, {})
+        self.restores = mock.Mock()
+        self.restores.resource_class = fakes.FakeResource(None, {})
+        self.services = mock.Mock()
+        self.services.resource_class = fakes.FakeResource(None, {})
+        self.transfers = mock.Mock()
+        self.transfers.resource_class = fakes.FakeResource(None, {})
         self.volume_encryption_types = mock.Mock()
         self.volume_encryption_types.resource_class = (
             fakes.FakeResource(None, {}))
-        self.restores = mock.Mock()
-        self.restores.resource_class = fakes.FakeResource(None, {})
-        self.qos_specs = mock.Mock()
-        self.qos_specs.resource_class = fakes.FakeResource(None, {})
-        self.availability_zones = mock.Mock()
-        self.availability_zones.resource_class = fakes.FakeResource(None, {})
-        self.transfers = mock.Mock()
-        self.transfers.resource_class = fakes.FakeResource(None, {})
-        self.services = mock.Mock()
-        self.services.resource_class = fakes.FakeResource(None, {})
-        self.quotas = mock.Mock()
-        self.quotas.resource_class = fakes.FakeResource(None, {})
-        self.quota_classes = mock.Mock()
-        self.quota_classes.resource_class = fakes.FakeResource(None, {})
-        self.consistencygroups = mock.Mock()
-        self.consistencygroups.resource_class = fakes.FakeResource(None, {})
-        self.cgsnapshots = mock.Mock()
-        self.cgsnapshots.resource_class = fakes.FakeResource(None, {})
-        self.auth_token = kwargs['token']
-        self.management_url = kwargs['endpoint']
-        self.capabilities = mock.Mock()
-        self.capabilities.resource_class = fakes.FakeResource(None, {})
-        self.pools = mock.Mock()
-        self.pools.resource_class = fakes.FakeResource(None, {})
+        self.volume_snapshots = mock.Mock()
+        self.volume_snapshots.resource_class = fakes.FakeResource(None, {})
+        self.volume_type_access = mock.Mock()
+        self.volume_type_access.resource_class = fakes.FakeResource(None, {})
+        self.volume_types = mock.Mock()
+        self.volume_types.resource_class = fakes.FakeResource(None, {})
+        self.volumes = mock.Mock()
+        self.volumes.resource_class = fakes.FakeResource(None, {})
 
 
 class TestVolume(utils.TestCommand):
@@ -596,6 +596,35 @@ class FakeBackup(object):
 
         return mock.Mock(side_effect=backups)
 
+    @staticmethod
+    def create_backup_record():
+        """Gets a fake backup record for a given backup.
+
+        :return: An "exported" backup record.
+        """
+
+        return {
+            'backup_service': 'cinder.backup.drivers.swift.SwiftBackupDriver',
+            'backup_url': 'eyJzdGF0dXMiOiAiYXZh',
+        }
+
+    @staticmethod
+    def import_backup_record():
+        """Creates a fake backup record import response from a backup.
+
+        :return: The fake backup object that was encoded.
+        """
+        return {
+            'backup': {
+                'id': 'backup.id',
+                'name': 'backup.name',
+                'links': [
+                    {'href': 'link1', 'rel': 'self'},
+                    {'href': 'link2', 'rel': 'bookmark'},
+                ],
+            },
+        }
+
 
 class FakeConsistencyGroup(object):
     """Fake one or more consistency group."""
