@@ -96,12 +96,12 @@ class CreateVolume(command.ShowOne):
         parser.add_argument(
             '--user',
             metavar='<user>',
-            help=_('Specify an alternate user (name or ID)'),
+            help=argparse.SUPPRESS,
         )
         parser.add_argument(
             '--project',
             metavar='<project>',
-            help=_('Specify an alternate project (name or ID)'),
+            help=argparse.SUPPRESS,
         )
         parser.add_argument(
             "--availability-zone",
@@ -159,7 +159,6 @@ class CreateVolume(command.ShowOne):
 
     def take_action(self, parsed_args):
         _check_size_arg(parsed_args)
-        identity_client = self.app.client_manager.identity
         volume_client = self.app.client_manager.volume
         image_client = self.app.client_manager.image
 
@@ -197,17 +196,25 @@ class CreateVolume(command.ShowOne):
             # snapshot size.
             size = max(size or 0, snapshot_obj.size)
 
-        project = None
+        # NOTE(abishop): Cinder's volumes.create() has 'project_id' and
+        # 'user_id' args, but they're not wired up to anything. The only way
+        # to specify an alternate project or user for the volume is to use
+        # the identity overrides (e.g. "--os-project-id").
+        #
+        # Now, if the project or user arg is specified then the command is
+        # rejected. Otherwise, Cinder would actually create a volume, but
+        # without the specified property.
         if parsed_args.project:
-            project = utils.find_resource(
-                identity_client.projects,
-                parsed_args.project).id
-
-        user = None
+            raise exceptions.CommandError(
+                _("ERROR: --project is deprecated, please use"
+                  " --os-project-name or --os-project-id instead."))
         if parsed_args.user:
-            user = utils.find_resource(
-                identity_client.users,
-                parsed_args.user).id
+            raise exceptions.CommandError(
+                _("ERROR: --user is deprecated, please use"
+                  " --os-username instead."))
+        if parsed_args.multi_attach:
+            LOG.warning(_("'--multi-attach' option is no longer supported by "
+                          "the block storage service."))
 
         volume = volume_client.volumes.create(
             size=size,
@@ -215,14 +222,11 @@ class CreateVolume(command.ShowOne):
             name=parsed_args.name,
             description=parsed_args.description,
             volume_type=parsed_args.type,
-            user_id=user,
-            project_id=project,
             availability_zone=parsed_args.availability_zone,
             metadata=parsed_args.property,
             imageRef=image,
             source_volid=source_volume,
             consistencygroup_id=consistency_group,
-            multiattach=parsed_args.multi_attach,
             scheduler_hints=parsed_args.hint,
         )
 
@@ -476,21 +480,13 @@ class MigrateVolume(command.Command):
             help=_("Enable generic host-based force-migration, "
                    "which bypasses driver optimizations")
         )
-        lock_group = parser.add_mutually_exclusive_group()
-        lock_group.add_argument(
+        parser.add_argument(
             '--lock-volume',
             action="store_true",
             help=_("If specified, the volume state will be locked "
                    "and will not allow a migration to be aborted "
                    "(possibly by another operation)")
         )
-        lock_group.add_argument(
-            '--unlock-volume',
-            action="store_true",
-            help=_("If specified, the volume state will not be "
-                   "locked and the a migration can be aborted "
-                   "(default) (possibly by another operation)")
-        )
         return parser
 
     def take_action(self, parsed_args):
@@ -563,6 +559,25 @@ class SetVolume(command.Command):
                    'in the database with no regard to actual status, '
                    'exercise caution when using)'),
         )
+        attached_group = parser.add_mutually_exclusive_group()
+        attached_group.add_argument(
+            "--attached",
+            action="store_true",
+            help=_('Set volume attachment status to "attached" '
+                   '(admin only) '
+                   '(This option simply changes the state of the volume '
+                   'in the database with no regard to actual status, '
+                   'exercise caution when using)'),
+        )
+        attached_group.add_argument(
+            "--detached",
+            action="store_true",
+            help=_('Set volume attachment status to "detached" '
+                   '(admin only) '
+                   '(This option simply changes the state of the volume '
+                   'in the database with no regard to actual status, '
+                   'exercise caution when using)'),
+        )
         parser.add_argument(
             '--type',
             metavar='<volume-type>',
@@ -649,6 +664,22 @@ class SetVolume(command.Command):
             except Exception as e:
                 LOG.error(_("Failed to set volume state: %s"), e)
                 result += 1
+        if parsed_args.attached:
+            try:
+                volume_client.volumes.reset_state(
+                    volume.id, state=None,
+                    attach_status="attached")
+            except Exception as e:
+                LOG.error(_("Failed to set volume attach-status: %s"), e)
+                result += 1
+        if parsed_args.detached:
+            try:
+                volume_client.volumes.reset_state(
+                    volume.id, state=None,
+                    attach_status="detached")
+            except Exception as e:
+                LOG.error(_("Failed to set volume attach-status: %s"), e)
+                result += 1
         if parsed_args.bootable or parsed_args.non_bootable:
             try:
                 volume_client.volumes.set_bootable(
